:- module(abschlussmodul). 		% Der Name des Modules wird definiert.

:- export(bachelorarbeit_erfolgreich_abgesch / 2).


:- use_module(student).
:- use_module(alle_Module).
:- use_module(basismodule).
:- use_module(vertiefungsmodule).
:- use_module(ergaenzungsmodule).
					% Hier Werden Modul geladen, die 
					% in diesem Modul aufgerufen werden.


:- lib(ic).	   				% Die ic-Bibliothek wird geladen.





/*
			Prdikaten-bersicht
	

			Abschlussmodul


	9. das_Abschlussmodul_abgesch(CP_bisher, Durchschnitt, CP_Gesamt)
		9.1 erreichte_CP_Basismodule_min_80 /1,
		9.2 erreichte_CP_Vertiefung  /1,
		9.3 erzielte_CP_Ergaenzung /1,		(Siehe EM 8.3)
		9.4 gesammt_CP_min_100 / 4,
		9.5 bachelorarbeit_erfolgreich_abgesch / 2. 
			9.5.1 oberseminar_abgeschlossen / 2.
			9.5.2 berechne_Durchschnitt_AM / 4.
		
*/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Prdikat 9.

das_Abschlussmodul_abgesch(CP_bisher, Durchschnitt, CP_Gesamt) :- 
	erzielte_CP_Basismodule_min_80(BM_Total),
	erzielte_CP_Vertiefung(VM_Total),		
	erzielte_CP_Ergaenzung(EM_Total),
	gesammt_CP_min_100(BM_Total, VM_Total, EM_Total, CP_bisher),
	bachelorarbeit_erfolgreich_abgesch(Durchschnitt, CP_Gesamt).
	
			% Hier werden alle Voraussetzungen definiert, die fr das
			% Abschliessen des Abschlussmoduls notwendig sind.

%
%%%%%%%%%%%%%%%%   Ende des Prdikats    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%       /* Im folgenden werden Klauseln definiert, die im Prdikat 9 benutzt wurden.*/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 9.1


erzielte_CP_Basismodule_min_80(BM_Total) :-
	schreibe_alle_erzielte_CP_basis(Liste),
	sum(Liste, BM_Total),
	BM_Total #>= 80.
			% Hier wird geprft, ob in Basismodulen mindestens 80 CP 
			% erreicht worden sind.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 9.2


erzielte_CP_Vertiefung(VM_Total) :-
	suche_alle_bestandene_Module_Vertiefung(List),
	pruefe_ob_alle_Vertiefungsmodule_regelgemaess_abgesch(List),
			% Hier werden alle bestandene Module in Vertiefungsmodul 
			% gesucht.

		(foreach(X, List),
		foreach(Y, T) do
		finde_CP_des_Moduls(X, Y)),
			% In der Liste T werden die erzielten CP gesamelt.

	sum(T, VM_Total).
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 9.4


gesammt_CP_min_100(A, B, C, D) :-
	Alle_Summen = [A, B, C],
	(foreach(X, Alle_Summen), 
	fromto(Expr,S1,S2,0) do 
		S1 = X + S2), 
	D #= eval(Expr).
	
			% Hier werden die erzielten CP in den Gebieten Basis-, Vertiefung- 
			% und Ergnzungsgebietes (A, B, C) zusammenadidiert und deren Summe 
			% der Variable D zugeordnet.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 9.5
	
bachelorarbeit_erfolgreich_abgesch(Durchschnitt, CP_Gesamt) :-
	oberseminar_abgeschlossen(CP_os, Z0),
	
	pruefungsversuch_BA(T),
	member(bachelorarbeit{modul_nr: M, noten:X, pruefungszeit:Z1}, T),

	das_Abschlussmodul(G),
	member(a_modul{modul_nr:M, cp:CP_ba}, G),  
	berechne_Durchschnitt_AM(X, Z0, Z1, Durchschnitt),
	CP_Gesamt is CP_os + CP_ba.	
			 
			% Hier wird geprft, ob und mit welcher Note der Student die Abschlussarbeit bestanden hat


%%%%%%%%%% 9.5.1

oberseminar_abgeschlossen(A, Z) :-
	pruefungsversuch_OS(W),
	member(oberseminar{modul_nr: B, note:N, pruefungszeit:Z}, W),
	N $=< 4.0,
	das_Abschlussmodul(Q),
	member(a_modul{modul_nr:B, cp:A}, Q).
			% Hier wird geprueft, ob das Oberseminar bereits 
			% abgeschlossen ist.



%%%%%%%%%% 9.5.2

berechne_Durchschnitt_AM(X, Z0, Z1, Durchschnitt) :-	
	
	Z0 #=< Z1,
			% Hier wird geprft, ob Oberseminar bereits abgeschlossen hat.
	length(X, K),			
 	1 #=(K #= 2 or K #= 3),
	
			% Hier wird vorausgesetzt, dass es nur zwei oder drei 
			% Benotungen geben kann.

	(K = 2	->
		(foreach(H, X) do
		 	H $=< 4.0)
		
	;
	true), 
			% Hier wird vorausgesetzt, dass nur dann zwei Benotungen 
			% ausreicehen, wenn keine der Beurteilungen eine 5.0 ist.
 
	(K = 3	->	
		X = [M1| Rest],
		Rest = [M2| _],
		-(M1, M2, Diff),
		abs(Diff, Abb),
		1 #= ((M1 $= 5.0 and M2 $=< 4.0) or (M1 $=< 4.0 and M2 $= 5.0) or (Abb $>= 2.0))
			
	;		
	true),	
			% Hier wird vorausgesetzt, dass nur dann eine dritte Beurteilung 
			% notwendig ist, wenn nur eine der ersten Beurteilungen eine 5.0
			% ist oder wenn die Differenz dieser mindestens 2.0 betrgt.
	sum(X, L),
	/(L, K, Durchschnitt),		
	Durchschnitt $=< 4.0.
			% Hier wird die Durchschnittsnote berechnet.




	

	











