:- module(basismodule). 		% Der Name des Modules wird definiert.

:- export(schreibe_alle_erzielte_CP_basis / 1).
:- export(ein_Pflichtmodul_basis_abgesch / 2).
:- export(berechne_Durchschnittsnote_Basismodule / 3).

			
:- use_module(student).
:- use_module(alle_Module).		% Hier Werden Module geladen, die 
					% in diesem Modul aufgerufen werden.


:- lib(ic).	   			% Die ic-Bibliothek wird geladen.



/*
			Prdikaten-bersicht
	

			BASISMODULE


	1. ein_Pflichtmodul_basis_abgesch(Modul, Note).
		1.1  das_Modul_ist_ein_Basismodul / 2,
		1.2  zulassungsvoraussetzung_erfuellt / 1,
			 1.2.1  einer_der_Studienleistungen_vorher_erbracht
			 1.2.2  studienleistung_vorher_erbracht_HWS(b_HWS_pr)
			 1.2.3  studienleistung_vorher_erbracht_M2( b_M2)
		1.3  not_endgueltig_nb / 1,
		1.4  modulabschlusspruefung_bestanden_basis / 2,
		
	2. das_Gebiet_Basismodul_abgesch(Liste).
		2.1  finde_nr_des_Moduls_BM / 1,
		2.2  pruefe_ob_alle_Basismodule_abgesch / 1,
			
	3. berechne_Durchschnittsnote_Basismodule(Durchschnittsnote, CP_Total, CP_benot ).
		3.1  schreibe_alle_erzielte_Noten_basis / 1,
		3.2  schreibe_alle_erzielte_CP_basis / 1,
		3.3  pruefe_ob_CP_Summe_min_98 / 2,
		3.4  gewichte_numerische_Noten_durch_CP_1 / 3,
		3.5  bilde_CP_Summe_der_numerischen_Noten / 3,
		
*/






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%			Prdikate bezglich  Basismodule            
%
%  Prdikat 1.


		/* Mit diesem Prdikat werden alle notwendige Bedingungen fr das 
		Abschliessen eines Basismoduls definiert. */

ein_Pflichtmodul_basis_abgesch(Modul, Note) :-
	
	das_Modul_ist_ein_Basismodul(Modul, _CP_Zahl),
			% Hier wird geprft, ob Modul ein der Datenbank bekannte 
			% Basismodule ist.

	zulassungsvoraussetzung_erfuellt(Modul),	
			% Hier wird geprft, ob Zulassungvoraussetzungen fr die 
			% Modulabschlussprfung erfllt sind.

	not_endgueltig_nb(Modul),
			% Der Student ist noch nicht in der Modulabschlussprfung 
			% drie mal durchgefallen bzw. endgltig nicht bestanden.

	modulabschlusspruefung_bestanden_basis(Modul, Note).
   


%%%%%%%%%%%%%%%%   Ende des Prdikats    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	  /* Im folgenden werden Klausel definiert, die in  Prdikat 1 benutzt wurden.*/


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.1


das_Modul_ist_ein_Basismodul(Modul, CP_Zahl) :-
	alle_basis_Module(W),
	member(b_modul{modul_nr:Modul, cp: CP_Zahl},W). 
			% Hier wird geprft, ob Modul ein Basismodul ist und wenn ja,
			% die CP des Modules wird der Variable CP_Zahl zugeordnet.       



        

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.2



zulassungsvoraussetzung_erfuellt(Modul) :-
	
	(Modul = b_PRG -> einer_der_Studienleistungen_vorher_erbracht(b_PRG, prog1, prog2)
	; true),

	(Modul = b_PROG_pr -> einer_der_Studienleistungen_vorher_erbracht(b_PROG_pr, prog1, prog2)
	; true),

	(Modul = b_HW ->  einer_der_Studienleistungen_vorher_erbracht(b_HW, edgi, hwr)
	; true),

	(Modul = b_GL ->  einer_der_Studienleistungen_vorher_erbracht(b_GL, gl1, gl2)
	; true),

	(Modul = b_HWS_pr ->  studienleistung_vorher_erbracht_HWS(b_HWS_pr)
	; true),

	(Modul = b_M2 ->  studienleistung_vorher_erbracht_M2( b_M2)
	; true).


		% Hier wird sechs mal geprft, ob die Variable Modul eine bestimmte
		% Modulnummer ist und jenach Nummer werden Klauseln aufgerufen.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.2.1


		/*  Vorausetzung fr die Modulabschlussprfungen  
		b_PRG,  b_PRG_pr,  b_HW  und  b_GL  */   


 einer_der_Studienleistungen_vorher_erbracht(Modulname, Studleist1, _) :-
	berechne_pruefungszeitpunkt(Modulname, Semester_Zahl1), 
	absolvierte_studienleistungen(W),
	member(studienleistung{vtg:Studleist1, pruefungszeit:Zeitpunkt}, W),
	Zeitpunkt #=< Semester_Zahl1, !.

 einer_der_Studienleistungen_vorher_erbracht(Modulname, _, Studleist2) :-
	berechne_pruefungszeitpunkt(Modulname, Semester_Zahl), 
	absolvierte_studienleistungen(W),
	member(studienleistung{vtg:Studleist2,  pruefungszeit:Zeitpunkt},W),
	Zeitpunkt #=< Semester_Zahl.	
		
		%  Hier wird geprft, ob eine der Sudienleistungen bereits absolviert ist.
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.2.2


		/*  Vorausetzung fr die Modulabschlusspr. b_HWS_pr  */

 studienleistung_vorher_erbracht_HWS(b_HWS_pr) :- 
	berechne_pruefungszeitpunkt(b_HWS_pr, Semester_Zahl), 
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_HW, pruefungszeit:Zeitpunkt, note:X},W),
	not(X = 5.0),
	Zeitpunkt #=< Semester_Zahl, !. 

 studienleistung_vorher_erbracht_HWS(b_HWS_pr) :- 
	berechne_pruefungszeitpunkt(b_HWS_pr, Semester_Zahl), 	
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_PRG, pruefungszeit:Zeitpunkt, note:X},W),
	not(X = 5.0),
	Zeitpunkt #=<Semester_Zahl, !.	
				
 studienleistung_vorher_erbracht_HWS(b_HWS_pr) :- 
	berechne_pruefungszeitpunkt(b_HWS_pr, Semester_Zahl), 
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_MOD, pruefungszeit:Zeitpunkt1, note:X1},W), 	
	member(pruefung_basis{modul_nr:b_DS, pruefungszeit:Zeitpunkt2, note:X2},W),
	not(X1 = 5.0),
	not(X2 = 5.0),
	Zeitpunkt1 #=< Semester_Zahl,
	Zeitpunkt2 #=< Semester_Zahl.	


		% Hier wird geprft, ob das Modul b_HW oder das Modul b_PRG oder
		% die beiden Module b_MOD und  b_DS abgeschlossen sind.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.2.3


		/*  Vorausetzung fr die Moodulabschlusspr. b_M2  */

 studienleistung_vorher_erbracht_M2( b_M2) :- 
	berechne_pruefungszeitpunkt(b_M2, Semester_Zahl),  
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_M1, pruefungszeit:Zeitpunkt, note:X},W),
	not(X = 5.0),
	Zeitpunkt #=<Semester_Zahl.

		% Hier wird geprft, ob das Modul b_M1 abgeschlossen ist.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.2.3.1

berechne_pruefungszeitpunkt(Modulname, Semester_Zahl) :-
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:Modulname,  pruefungszeit:Y},W),
	Semester_Zahl is Y.
			
	% Mit dieser klausel wird die Zeit, in der die  Modulabschlusprfung 
	%  gemacht worden ist, der Variable Semester_Zahl zugeordnet. 






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.3

not_endgueltig_nb(Modul) :-
	pruefungsversuche_BM(W1),
	(foreach(Z1, W1), fromto(W2, Out, In, []), param(Modul)  do
     		arg(1, Z1, Modul) -> Out = [Z1|In]
				; 
				Out = In), 
	
			% Hier werden alle Prfungsversuche in dem selben Modul
			% rausgefiltert und in die Liste W2 geschrieben.
 	
	(foreach(Z2, W2), fromto(W3, Out, In, []) do
     	arg(3, Z2, 5.0) -> Out = [Z2|In]; Out = In), 
	length(W3, D),
	D#<  3.
			% Hier werden alle fehlgeschlagene Prfungsversuche 
			% rausgefiltert und in die Liste W3 geschrieben, mit der 
			% Bedingung dass, W3 weniger als 3 Elemente enthalten darf.  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 1.4

modulabschlusspruefung_bestanden_basis(Modul, N) :-
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:Modul, note:N}, W),
	not(N = 5.0) .   
		
			% Mit dieser Klausel wird geprft, ob der Student die 
			% Modulabschlussprfung bestanden hat und wenn ja, 
			% die Note wird der Variable N zugeordnet.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Prdikat 2.


das_Gebiet_Basismodul_abgesch(Liste) :-
	finde_alle_Modul_nr_BM(Liste),
			% Hier werden alle Modulnummern der Basismodule der 
			% Variable Liste zugeordnet.
	pruefe_ob_alle_Basismodule_abgesch(Liste).
			% Hier wird fr jedes Modul geprft, ob Student das Modul
			% abgeschlossen hat.

%
%%%%%%%%%%%%%%%%   Ende des Prdikats    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	 
/* Im folgenden werden Klauseln defniert, die im  Prdikat 2. benutzt wurden.*/



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 2.1
 
finde_alle_Modul_nr_BM(Liste) :- 
	alle_basis_Module(Y),		% Hier wird der Variable Y die Struktur
					% zugeordnet, die alle Basismodule enthlt.

	(foreach(X1, Y),  foreach(P1, Temp)	% Hier wird iterativ die Modulnummer 
	do finde_nr_des_Moduls_BM(X1, P1)),	% eines jeden Basismoduls  in der Liste
					% Temp geschrieben. 	
 
	sort(Temp, Liste).			% Hier werden verdopplungen eliminiert.
				
			% Mit dieser Klausel werden alle Modulnummer der 
			% Basismodule gesammelt, und der Variable Liste zugeordnet.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 2.1.1

finde_nr_des_Moduls_BM(b_modul{modul_nr:X}, Z) :-
	Z = X.
		
		
	/* Mit dieser Klausel wird der Variable Z die Modulnummer (X) zugeordnet.  */



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 2.2


pruefe_ob_alle_Basismodule_abgesch(Liste) :-
	(foreach(X, Liste)  do 
	ein_Pflichtmodul_basis_abgesch(X, _Note)).
			% Hier wird fr jede Modulnummer in der Liste iterativ 
			% geprft, ob der Student die einzelnen Module 
			% abgeschlossen hat. 
      


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Prdikat  3.


berechne_Durchschnittsnote_Basismodule(Durchschnittsnote, CP_Total, CP_benot ) :- 
	
	das_Gebiet_Basismodul_abgesch(_Liste),
	
	schreibe_alle_erzielte_Noten_basis(T1),
			% In T1 werden alle Noten geschrieben.

	schreibe_alle_erzielte_CP_basis(T2),
			% In T2 werden alle CP geschrieben.
	pruefe_ob_CP_Summe_min_98(T2, CP_Total),
			% Hier wird geprft, ob 98 CP in 
			% Basismodulen erzieltt worden sind. 	

	gewichte_numerische_Noten_durch_CP_1(T1, T2, Gesamt),
			% Hier werden numerische Noten durch CP gewichtet
			%  und die Summe wird der Variable Gesamt zugeordnet.

	bilde_CP_Summe_der_numerischen_Noten(T1, T2, CP_benot),		
			% Hier werden die CP der  numerischen Noten addiert und
			% der der Variable CP_benot zugeordnet.
		
	Durchschnittsnote is Gesamt / CP_benot,
			% Mit dem eingebauten Prdikat is / 3 wird der Durchschnitt
			% berechnet.

	print('Durchschnittsnote : '), 
	print(Durchschnittsnote), nl.
			% Hier wird das Ergebnis ausgegeben.

%
%%%%%%%%%%%%%%%%   Ende des Prdikates         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


	/* Im folgenden werden Klauseln definiert, die im Prdikat 3. benutzt wurden.*/


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 3.1


schreibe_alle_erzielte_Noten_basis(T) :-
	
	finde_alle_Modul_nr_BM(Liste),

	(foreach(X, Liste),  foreach(P, T) do  
		modulabschlusspruefung_bestanden_basis(X, P)).		
				% Alle  benoteten und unbenoteten  Noten
				% werden in die Liste T geschrieben.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 3.2


schreibe_alle_erzielte_CP_basis(T) :-
	finde_alle_Modul_nr_BM(Liste),
	(foreach(X, Liste), 
	foreach(P, T) do
		das_Modul_ist_ein_Basismodul(X, P)).
				% Alle erzielten CP werden in die Liste T geschrieben.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 3.3


pruefe_ob_CP_Summe_min_98(T2, CP_Total) :-
	sum(T2, CP_Total),	
	CP_Total #>= 98.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 3.4


gewichte_numerische_Noten_durch_CP_1(T1, T2, Gesamt) :-
	(foreach(X, T1), 
	foreach(Y, T2),
	foreach(Z, T3) do
		real(X) -> *(X, Y, Z) 
		; 
		Z =0),
	sum(T3, Gesamt).	
				% Alle numerischen Noten werden mit CP  
				% gewichtet  und die Summe wird der Variable
				% Gesamt zugeordnet. 
			



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 3.5


bilde_CP_Summe_der_numerischen_Noten(T1, T2, CP_benot) :-		
	(foreach(X, T1), 
	foreach(Y, T2),
	foreach(Z, T4) do
		real(X) -> Z = Y  
		; 
		Z =0),
	sum(T4, CP_benot). 
				% Nicht numerische Noten bekommen den CP-Wert
				%  0 und die restlichen werden zusammenaddiert.










