:- module(ergaenzungsmodule).		% Der Name des Modules wird definiert.


:- export(erzielte_CP_Ergaenzung /1).
					% Hier wird das Praedikat ausserhalb des Modules
					% Sichtbar gemacht.
	

:- use_module(student).
:- use_module(alle_Module).
:- use_module(basismodule).
					% Hier Werden Module geladen, die 
					% in diesem Modul aufgerufen werden.

:- lib(ic).
			% Hier wird die IC-Bibliothek galaden.


/*
			Praedikaten-Uebersicht
	

			Ergaenzungsmodul

		8. ergaenzungsmodul_abgeschlossen(Modul, CP_Total).
		8.1 voraussetzung_Ergaenzungsmodul / 1,
		8.2 voraussetzung_wenn_X_tutorium / 2,
		8.2.1 bereits_abgesch / 2,
		8.3 erzielte_CP_Ergaenzung / 1,
		8.3.1 berechne_CP_Total / 2,



*/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%            Prdikate bezglich  Ergnzungssmodule               %%%%%%%%
%
%  Prdikat 8.

ergaenzungsmodul_abgeschlossen(Modul, CP_Total) :-
	erfolgreiche_pruefungsversuche_EM(W),
	member(ergaenzungsmodul{modul_nr: Modul, pruefungszeit: Z}, W),
	voraussetzung_Ergaenzungsmodul(Z),
	voraussetzung_wenn_X_tutorium(Modul, Z),
	erzielte_CP_Ergaenzung(CP_Total).
			% Hier werden alle Voraussetzungen definiert, die fuer 
			% das Abschliessen der Ergnzungsmodule notwendig sind.


%
%%%%%%%%%%%%%%%%   Ende der Klausel    %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



 	 /* Im folgenden werden 8 Klauseln definiert, die im Prdikat 8 benutzt wurden.*/


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 8.1


voraussetzung_Ergaenzungsmodul(Zeit) :- 
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_HW, pruefungszeit:Zeitpunkt},W),
	Zeitpunkt #=< Zeit, !.  		
			% Hier wird geprueft, ob das Modul b_HW bereits abgeschlossen ist.

voraussetzung_ergaenzung(Zeit) :- 
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_PRG, pruefungszeit:Zeitpunkt},W),
	Zeitpunkt #=< Zeit, !.		
			% Hier wird geprueft, ob das Modul b_PRG bereits abgeschlossen ist.

voraussetzung_ergaenzung(Zeit) :-
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:b_MOD, pruefungszeit:Zeitpunkt1},W), 	
	member(pruefung_basis{modul_nr:b_DS, pruefungszeit:Zeitpunkt2},W),
	Zeitpunkt1 #=< Zeit,		
	Zeitpunkt2 #=< Zeit.		
			% Hier wird geprueft, ob beide Module (b_MOD und b_DS)
			% bereits abgeschlossen sind.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 8.2


voraussetzung_wenn_X_tutorium(X, Z) :-
	(X =[b_TL, MOD]	->
		bereits_abgesch(MOD, Z)
	;
		member( X, [b_NMG, b_PM])
	).

			% Hier geprueft, ob der Student eine Tutoriumleistung im
			% Ergaenzungsmodul abgeschlossen hat. Wenn ja, muss
			% der Student das entsprechende Modul abgeschlossen haben.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 8.2.1

bereits_abgesch(Modul, Z1) :-
	pruefungsversuche_BM(W),
	member(pruefung_basis{modul_nr:Modul, pruefungszeit:Z2, note:N}, W),
	ein_Pflichtmodul_basis_abgesch(Modul, _Note),
	N $=< 4.0,
	Z2 #=< Z1, !.
			

bereits_abgesch(Modul, Z1) :-
	pruefungsversuche_VM(W),
	member(pruefung_vertiefung{modul_nr:Modul, pruefungszeit:Z2, note:N}, W), 
	N $=< 4.0,
	Z2 #=< Z1.

			% Hier wird geprueft, ob der Stundent das entsprechende Modul
			% im Basis- oder Vertiefungsmodule abgeschlossen hat.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 8.3


erzielte_CP_Ergaenzung(Total) :-
	erfolgreiche_pruefungsversuche_EM(W),
	(foreach(X, W), 
	foreach(Y, L) do
		berechne_CP_Total(X, Y)
		
	),
	sum(L, Total).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% 8.3.1


berechne_CP_Total(ergaenzungsmodul{modul_nr:P, pruefungszeit: 5}, Y) :-
	alle_Ergaenzungsmodule(R),
	(P = [b_TL|_]	->
		
		member(erg_modul{modul_nr:b_TL, cp:Y}, R)
	;
		(P = Modul),
		member(erg_modul{modul_nr:Modul, cp:Y}, R)

	).
		
			% Hier wird das erreichte CP gerechnet und geprft, fr den Fall dass der Student 
	
			% Tutoriumleistung gemacht hat, ob er das entsprechende Modul abgeschlossen hat.